/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guiaccountinformation.h"
#include <qlistview.h>
#include <aqmoney2/account.h>
#include <openhbci2/pointer.h>
#include <openhbci2/bank.h>

/**
* this is the constructor of class for operations for account/and accountinformations referring to guimain
* @param parent the parent dialog of this window
* @param hobit
* @param name widgets title
*/
GuiAccountInformation::GuiAccountInformation(QWidget * parent, GuiMain * guimain, HoBIT* hobit, const char *name)
    :GuiAccountInformationBase(parent, name)
{
  _hobit = hobit;
  _main = guimain;
  qDebug("Constructor GuiAccountInformation");
}


/**
 * destructor
*/
GuiAccountInformation::~GuiAccountInformation()
{
  qDebug("Destructor GuiAccountInformation");
}



/*!
    \fn GuiAccountInformation::refresh()
 */

/**
* get/refresh information about current account
*/
void GuiAccountInformation::refresh()
{
  listViewAccount->setSorting(-1); //switch off sorting
  listViewAccount->clear();
  Pointer<Account> account;
  QString bankName = tr("unknown institute");

  QString noted;
  QString booked;
  QString creditLine;


  // check if guimain is with an account, if not find and set an account
  if (_main->getCurrentAccount() != NULL)
    account = _hobit->findAccount(_main->getCurrentAccount().ascii());

  if (account.isValid())
  {

    QString accountName(account.ref().accountName().c_str());
    if (accountName.isEmpty()) accountName = tr("no description selected");

    string bankId = account.ref().bankCode();
    int country = account.ref().country();

    Pointer<Bank> bank = _hobit->findBank(country, bankId);

    if (bank.isValid())
      bankName = QString(bank.ref().name().c_str());

    // get Balances
    noted = QString(account.ref().notedBalance().value().toReadableString().c_str());
    booked = QString(account.ref().bookedBalance().value().toReadableString().c_str());
    creditLine = QString(account.ref().creditLine().toReadableString().c_str());

    new QListViewItem(listViewAccount, tr("Credit Line:"), creditLine);
    new QListViewItem(listViewAccount, tr("Noted Balance:"), noted);
    new QListViewItem(listViewAccount, tr("Booked Balance:"), booked);
    new QListViewItem(listViewAccount, "", "");
    new QListViewItem(listViewAccount, tr("Bank Id:"), QString(bankId.c_str()));
    new QListViewItem(listViewAccount, tr("Bank Name:"), bankName);
    new QListViewItem(listViewAccount, tr("HBCI User Id:"), QString(account.ref().customer().c_str()));
    new QListViewItem(listViewAccount, tr("Account Id:"), QString(account.ref().accountId().c_str()));
    new QListViewItem(listViewAccount, tr("Account Description:"), accountName);
    new QListViewItem(listViewAccount, tr("Account Owner:"), QString(account.ref().userName().c_str()));
  }
}

/**
* delete account referring to guimain
*/
void GuiAccountInformation::slotDeleteAccount()
{
}

/**
* add account referring to guimain
*/
void GuiAccountInformation::slotAddAccount()
{
  _main->slotAddAccount();
}

/**
* change account referring to guimain
*/
void GuiAccountInformation::slotChangeAccountName()
{
  _main->changeAccountName();
}

/**
* switch account referring to guimain
*/
void GuiAccountInformation::slotSwitchAccount()
{
  _main->switchAccount();
}

