/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef GUIHELPWINDOW_H
#define GUIHELPWINDOW_H
#include <qtextbrowser.h>
#include <qstringlist.h>
#include <qmap.h>
#include <qdir.h>

#include "guihelpwindowbase.h"

class QComboBox;
class QPopupMenu;


/**
The help browser. Based on help window example by Trolltech.
*
*/
class GuiHelpWindow : public GuiHelpWindowBase
{
  Q_OBJECT

public:

  /**
  *constructor of GuiHelpWindow
  *
  *@param home_ QString
  *@param _path QStringList
  *@param parent pointer on current QWidget
  *@param name name of the Widget
  */
  GuiHelpWindow(QWidget* parent, const char *name, const QString& home_,  const QStringList& path);

  /**
  *destructor of GuiHelpWindow
  */
  ~GuiHelpWindow();

  /**
  *Set the helpfile to be displayed
  *@param helfFile name of help file
  */
  void setHelpfile(const QString & helpFile);

signals:

  void cancelled(int);

private slots:

  /**
  *function for a source changing
  *@param url QString with the location of the new source
  */
  void sourceChanged( const QString& );

  
  /**
  *function for opening a file
  */
  void openFile();

  /**
  *function for printing
  */
  void print();

  /**
  *function for the path selection
  *@param _path path_location
  */
  void pathSelected( const QString & );

  /**
  *function for chosing history
  *@param i number of chosen history
  */
  void histChosen( int );

  /**
  *function for chosing bookmark
  *@param i number of chosen bookmark
  */
  void bookmChosen( int );

  /**
  *function for adding a bookmark
  */
  void addBookmark();

  /**
  *Fires signal cancelled
  */
  void slotBackToProgram();

private:

  /**
  *function for reading history
  */
  void readHistory();

  /**
  *function for reading the bookmarks
  */
  void readBookmarks();

  int backwardId, forwardId;
  QStringList history, bookmarks;
  QMap<int, QString> mHistory, mBookmarks;
  QPopupMenu *hist, *bookm;
};





#endif
