/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch                                    *
 *   marcel@linux                                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#include "guitransactioninfo.h"
#include <qdatetime.h>
#include <qlabel.h>
#include <qstring.h>

GuiTransactionInfo::GuiTransactionInfo(QWidget * parent, GuiMain *main, HoBIT* hobit)
 : GuiTransactionInfoBase(parent){
	_main = main;  
	_hobit = hobit;
   qDebug("Constructor GuiTransactionInfo");   
}


GuiTransactionInfo::~GuiTransactionInfo()
{
  qDebug("Destructor GuiTransactionInfo");
}

void GuiTransactionInfo::slotBack(){
emit goBack();
}

void GuiTransactionInfo::showStandingOrder(Pointer<StandingOrder> sto){

   textLabelType->setText(tr("Standing Order"));
   textLabelPrimanota->hide();
   labelPrimanota->hide();
   
   clearAll();

   // own account
   textLabelAccountIdFrom->setText(sto.ref().accountId().c_str());
   textLabelBankIdFrom->setText(sto.ref().bankCode().c_str());
   textLabelCountryFrom->setText(QString::number(sto.ref().country()));
   
   // other account
   textLabelOwnerTo->setText(sto.ref().otherName().c_str());
   textLabelAccountIdTo->setText(sto.ref().otherAccountId().c_str());
   textLabelBankIdTo->setText(sto.ref().otherBankId().c_str());
   textLabelCountryTo->setText(QString::number(sto.ref().otherCountry()));
   
   
   textLabelAmount->setText(sto.ref().value().toReadableString().c_str());
   label3->setText(tr("order number"));
    QString jobid = sto.ref().jobId().c_str();
   textLabelThird->setText(QString::number(jobid.toInt()));
   
   Date d = sto.ref().firstDate();
   QDate dateFirst(d.year(), d.month(), d.day());
   d = sto.ref().lastDate();
   QDate dateLast(d.year(), d.month(), d.day());

   label1->setText(tr("first execution"));
   textLabelFirst->setText(dateFirst.toString());
   label2->setText(tr("last execution"));
   textLabelSecond->setText(dateFirst.toString());
   
   QString purpose = makePurposeString(sto.ref().purposes());
   textLabelPurpose->setText(purpose);

}

void GuiTransactionInfo::showTransaction(Pointer<Transaction> trc){

   textLabelType->setText(tr("Transaction"));
   textLabelPrimanota->show();
   labelPrimanota->show();
   
   clearAll();
   
    // own account
   textLabelAccountIdFrom->setText(trc.ref().ouraccountId().c_str());
   textLabelBankIdFrom->setText(trc.ref().ourbankcode().c_str());
 //  textLabelCountryFrom->setText(QString::number(sto.ref().ourcountry()));
   
   // other account
   textLabelOwnerTo->setText(trc.ref().othername().c_str());
   textLabelAccountIdTo->setText(trc.ref().otheraccountId().c_str());
   textLabelBankIdTo->setText(trc.ref().otherbankcode().c_str());
//   textLabelCountryTo->setText(QString::number(sto.ref().othercountry()));

   QString purpose = makePurposeString(trc.ref().purposeLines());
   textLabelPurpose->setText(purpose);
   
   Date d = trc.ref().date();
   QDate date1(d.year(), d.month(), d.day());
   d = trc.ref().storeDate();
   QDate date2(d.year(), d.month(), d.day());  
   d = trc.ref().valutaDate();
   QDate date3(d.year(), d.month(), d.day());  
       
   label1->setText(tr("date"));
   textLabelFirst->setText(date1.toString());
   label2->setText(tr("store date"));
   textLabelFirst->setText(date2.toString());
   label3->setText(tr("valuta date"));
   textLabelFirst->setText(date3.toString());
   
   textLabelPrimanota->setText(trc.ref().primanota().c_str());
   textLabelAmount->setText(trc.ref().value().toReadableString().c_str());
   
}


/**
*Set Current used Account to Label in Formular 
*/
void GuiTransactionInfo::refresh()
{
	Pointer<Account> account;
        
        if (_main->getCurrentAccount() != "") {
	account = _hobit->findAccount(_main->getCurrentAccount().ascii());
	
	if (account.isValid()){
	     textLabelOwnerFrom->setText(account.ref().userName().c_str());
	}
        else textLabelOwnerFrom->setText(tr("No username found"));
	}
}

/**
* Clears all textfields
*/
void GuiTransactionInfo::clearAll()
{
// own account
   textLabelAccountIdFrom->setText("");
   textLabelBankIdFrom->setText("");
   textLabelCountryFrom->setText("");
   
   // other account
   textLabelOwnerTo->setText("");
   textLabelAccountIdTo->setText("");
   textLabelBankIdTo->setText("");
   textLabelCountryTo->setText("");

    
   textLabelPurpose->setText("");
      
   textLabelFirst->setText("");
   textLabelFirst->setText("");
   textLabelFirst->setText("");
   
   textLabelPrimanota->setText("");
   textLabelAmount->setText("");
	
}


/** creates purpose lines */
QString GuiTransactionInfo::makePurposeString(list<string> purposeLines){

 QString purpose;

 list <string>::iterator it;

 for (it = purposeLines.begin(); it != purposeLines.end(); it++){
          if(!(*it).empty()) {
	  	purpose+= QString((*it).c_str()) + "\n";
	  }
     }

  purpose.truncate(purpose.length() -1); //remove last \n
  return purpose;
}