/***************************************************************************
 *   Copyright (C) 2004 by Walter Roth                                     *
 *   walter@amd2600-roth                                                   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/


#include <list>
#include <string>
using namespace std;

#include <openhbci2.h>
#include <openhbci2/value.h>
using namespace HBCI;

#include <qlistview.h>
#include <qstring.h>
#include <qradiobutton.h>
#include <qpushbutton.h>

#include "guitransfertemplates.h"
#include "hobit_transfertemplate.h"
#include "globals.h"

GuiTransferTemplates::GuiTransferTemplates(GuiMain* main, QWidget* parent, const char* name, HoBIT* hobit)
: GuiTransferTemplatesBase(parent,name)
{
 _hobit = hobit;
 _main = main;
  deleteButton->setEnabled(false);
  useButton->setEnabled(false);
  refresh();
}

GuiTransferTemplates::~GuiTransferTemplates()
{
  qDebug("Destructor GuiTransferTemplates finished");
}

/*$SPECIALIZATION$*/
void GuiTransferTemplates::select()
{
  QListViewItem* item = listViewTemplates->selectedItem();
  
  if (item != 0){
  deleteButton->setEnabled(true);
  useButton->setEnabled(true);
  }
  else{
  deleteButton->setEnabled(false);
  useButton->setEnabled(false);
  }
  
  
}

void GuiTransferTemplates::use()
{
QListViewItem* item; 
item = listViewTemplates->selectedItem();
QString name =  item->text(0);
QString account =  item->text(1);
QString bank =  item->text(2);
QString value =  item->text(3);
QString purpose =  item->text(4);
QString purpose2 =  item->text(5);
value.truncate(value.length() - 4);

if (radioButtonSingleTransfer->isChecked()){
_main->slotRaiseTemplate(SINGLE_TRANSFER_WIDGET, name, account, bank, value,  purpose, purpose2);
}
if (radioButtonDatedTransfer->isChecked()){
_main->slotRaiseTemplate(DATED_SINGLE_TRANSFER_WIDGET, name, account, bank, value, purpose, purpose2);
}
if (radioButtonStandingOrder->isChecked()){
_main->slotRaiseTemplate(START_STANDING_ORDER_WIDGET, name, account, bank, value, purpose, purpose2);
}

}

void GuiTransferTemplates::remove()
{
QListViewItem* item; 
item = listViewTemplates->selectedItem();
QString name =  item->text(0);
QString account =  item->text(1);
QString bank =  item->text(2);
QString value =  item->text(3);
QString purpose1 =  item->text(4);
QString purpose2 =  item->text(5);



_hobit->deleteTransferTemplate(_main->getCurrentAccount().latin1(), 
                               name.latin1(), 
                               account.latin1(), 
                               bank.latin1(), 
                               value.latin1(), 
                               purpose1.latin1(), 
                               purpose2.latin1());
refresh();
}

void GuiTransferTemplates::cancel()
{
  _main->slotShowLastVisibleWidget();
}


void GuiTransferTemplates::refresh()
{
  
  _templates = _hobit->getTransferTemplates();
  list<Pointer<HoBIT_TransferTemplate> >::iterator it;
  string currentAccount = _main->getCurrentAccount();
  
  listViewTemplates->clear();
  for(it = _templates.begin(); it != _templates.end(); it++){

  string account = (*it).ref().ownAccountId();
  if (account == currentAccount)
  {
  string name = (*it).ref().otherName();
  string account = (*it).ref().otherAccountId();
  string bank = (*it).ref().otherBank();
  Value value = (*it).ref().value();
  string purpose = (*it).ref().purpose1();
  string purpose2 = (*it).ref().purpose2();
  
  new QListViewItem(	listViewTemplates, 
  			name.c_str(),
			account.c_str(),
			bank.c_str(),
			value.toReadableString().c_str(),
			purpose.c_str(),
			purpose2.c_str());
  }			
  
  }
  
  select();
  
}



