/***************************************************************************
 *   Copyright (C) 2004 by Marcel Mulch                                    *
 *   marcel@mulch-online.de                                                *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/
#ifndef HOBIT_TRANSFERTEMPLATE_H
#define HOBIT_TRANSFERTEMPLATE_H

#include <string>
using namespace std;

#include <openhbci2.h>
#include <openhbci2/value.h>
using namespace HBCI;

/**
@author Marcel Mulch
*/
class HoBIT_TransferTemplate{
public:
    /** base constructor creating 
      * a new transfer template
      */
    HoBIT_TransferTemplate(const string& ownAccountId,
    			   const string& otherName, 
			   const string& otherAccountId,
			   const string& otherBank,
			   const Value& value,
			   const string purpose1,
			   const string purpose2);
			   
   
   /** Constructor which creates a new transfer template
    * from a gwen node - needed for reading out the config 
    * file 
    */
   HoBIT_TransferTemplate(GWEN_DB_NODE *db);

     /** base destructor 
     */
    ~HoBIT_TransferTemplate();
    
    /** writes a single database entry. 
      * stores a single transfertemplate 
      */
    void toConfig(GWEN_DB_NODE *db);
    
    /** reads a single transfer template entry 
      * from the database
      */
    void fromConfig(GWEN_DB_NODE *db);
    
    
    // getter methods
    const string& ownAccountId(){return _ownAccountId;}
    const string& otherName(){return _otherName;}
    const string& otherAccountId(){return _otherAccountId;}
    const string& otherBank(){return _otherBank;}
    const Value& value(){return _value;}
    const string& purpose1() {return _purpose1;}
    const string& purpose2() {return _purpose2;}
    
    // setter methods
    void setOwnAccountId(const string& accountId){_ownAccountId = accountId;}
    void setOtherName(const string& name){_otherName = name;}
    void setOtherAccountId(const string& accountId){_otherAccountId = accountId;}
    void setOtherBank(const string& bank){_otherBank = bank;}
    void setValue(const Value& val){_value = val;}
    void setPurpose1(const string& purp){_purpose1 = purp;}
    void setPurpose2(const string& purp){_purpose2 = purp;}

private:
    string _ownAccountId;
    string _otherName;
    string _otherAccountId;
    string _otherBank;
    Value _value;
    string _purpose1;
    string _purpose2;
};



#endif
